# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..core.request_options import RequestOptions
from ..core.http_response import HttpResponse
from ..types.get_dispute_response import GetDisputeResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..types.accept_dispute_response import AcceptDisputeResponse
from ..requests.create_dispute_evidence_file_request import CreateDisputeEvidenceFileRequestParams
from .. import core
from ..types.create_dispute_evidence_file_response import CreateDisputeEvidenceFileResponse
import json
from ..types.dispute_evidence_type import DisputeEvidenceType
from ..types.create_dispute_evidence_text_response import CreateDisputeEvidenceTextResponse
from ..types.submit_evidence_response import SubmitEvidenceResponse
from ..core.client_wrapper import AsyncClientWrapper
from ..core.http_response import AsyncHttpResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawDisputesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(
        self, dispute_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetDisputeResponse]:
        """
        Returns details about a specific dispute.

        Parameters
        ----------
        dispute_id : str
            The ID of the dispute you want more details about.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetDisputeResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/disputes/{jsonable_encoder(dispute_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetDisputeResponse,
                    construct_type(
                        type_=GetDisputeResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def accept(
        self, dispute_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[AcceptDisputeResponse]:
        """
        Accepts the loss on a dispute. Square returns the disputed amount to the cardholder and
        updates the dispute state to ACCEPTED.

        Square debits the disputed amount from the seller’s Square account. If the Square account
        does not have sufficient funds, Square debits the associated bank account.

        Parameters
        ----------
        dispute_id : str
            The ID of the dispute you want to accept.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AcceptDisputeResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/disputes/{jsonable_encoder(dispute_id)}/accept",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AcceptDisputeResponse,
                    construct_type(
                        type_=AcceptDisputeResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_evidence_file(
        self,
        dispute_id: str,
        *,
        request: typing.Optional[CreateDisputeEvidenceFileRequestParams] = OMIT,
        image_file: typing.Optional[core.File] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateDisputeEvidenceFileResponse]:
        """
        Uploads a file to use as evidence in a dispute challenge. The endpoint accepts HTTP
        multipart/form-data file uploads in HEIC, HEIF, JPEG, PDF, PNG, and TIFF formats.

        Parameters
        ----------
        dispute_id : str
            The ID of the dispute for which you want to upload evidence.

        request : typing.Optional[CreateDisputeEvidenceFileRequestParams]

        image_file : typing.Optional[core.File]
            See core.File for more documentation

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateDisputeEvidenceFileResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/disputes/{jsonable_encoder(dispute_id)}/evidence-files",
            method="POST",
            data={},
            files={
                **(
                    {"request": (None, json.dumps(jsonable_encoder(request)), "application/json; charset=utf-8")}
                    if request is not OMIT
                    else {}
                ),
                **(
                    {"image_file": core.with_content_type(file=image_file, default_content_type="image/jpeg")}
                    if image_file is not None
                    else {}
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateDisputeEvidenceFileResponse,
                    construct_type(
                        type_=CreateDisputeEvidenceFileResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_evidence_text(
        self,
        dispute_id: str,
        *,
        idempotency_key: str,
        evidence_text: str,
        evidence_type: typing.Optional[DisputeEvidenceType] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateDisputeEvidenceTextResponse]:
        """
        Uploads text to use as evidence for a dispute challenge.

        Parameters
        ----------
        dispute_id : str
            The ID of the dispute for which you want to upload evidence.

        idempotency_key : str
            A unique key identifying the request. For more information, see [Idempotency](https://developer.squareup.com/docs/working-with-apis/idempotency).

        evidence_text : str
            The evidence string.

        evidence_type : typing.Optional[DisputeEvidenceType]
            The type of evidence you are uploading.
            See [DisputeEvidenceType](#type-disputeevidencetype) for possible values

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateDisputeEvidenceTextResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/disputes/{jsonable_encoder(dispute_id)}/evidence-text",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "evidence_type": evidence_type,
                "evidence_text": evidence_text,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateDisputeEvidenceTextResponse,
                    construct_type(
                        type_=CreateDisputeEvidenceTextResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def submit_evidence(
        self, dispute_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[SubmitEvidenceResponse]:
        """
        Submits evidence to the cardholder's bank.

        The evidence submitted by this endpoint includes evidence uploaded
        using the [CreateDisputeEvidenceFile](api-endpoint:Disputes-CreateDisputeEvidenceFile) and
        [CreateDisputeEvidenceText](api-endpoint:Disputes-CreateDisputeEvidenceText) endpoints and
        evidence automatically provided by Square, when available. Evidence cannot be removed from
        a dispute after submission.

        Parameters
        ----------
        dispute_id : str
            The ID of the dispute for which you want to submit evidence.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SubmitEvidenceResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/disputes/{jsonable_encoder(dispute_id)}/submit-evidence",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SubmitEvidenceResponse,
                    construct_type(
                        type_=SubmitEvidenceResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawDisputesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(
        self, dispute_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetDisputeResponse]:
        """
        Returns details about a specific dispute.

        Parameters
        ----------
        dispute_id : str
            The ID of the dispute you want more details about.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetDisputeResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/disputes/{jsonable_encoder(dispute_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetDisputeResponse,
                    construct_type(
                        type_=GetDisputeResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def accept(
        self, dispute_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[AcceptDisputeResponse]:
        """
        Accepts the loss on a dispute. Square returns the disputed amount to the cardholder and
        updates the dispute state to ACCEPTED.

        Square debits the disputed amount from the seller’s Square account. If the Square account
        does not have sufficient funds, Square debits the associated bank account.

        Parameters
        ----------
        dispute_id : str
            The ID of the dispute you want to accept.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AcceptDisputeResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/disputes/{jsonable_encoder(dispute_id)}/accept",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AcceptDisputeResponse,
                    construct_type(
                        type_=AcceptDisputeResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_evidence_file(
        self,
        dispute_id: str,
        *,
        request: typing.Optional[CreateDisputeEvidenceFileRequestParams] = OMIT,
        image_file: typing.Optional[core.File] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateDisputeEvidenceFileResponse]:
        """
        Uploads a file to use as evidence in a dispute challenge. The endpoint accepts HTTP
        multipart/form-data file uploads in HEIC, HEIF, JPEG, PDF, PNG, and TIFF formats.

        Parameters
        ----------
        dispute_id : str
            The ID of the dispute for which you want to upload evidence.

        request : typing.Optional[CreateDisputeEvidenceFileRequestParams]

        image_file : typing.Optional[core.File]
            See core.File for more documentation

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateDisputeEvidenceFileResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/disputes/{jsonable_encoder(dispute_id)}/evidence-files",
            method="POST",
            data={},
            files={
                **(
                    {"request": (None, json.dumps(jsonable_encoder(request)), "application/json; charset=utf-8")}
                    if request is not OMIT
                    else {}
                ),
                **(
                    {"image_file": core.with_content_type(file=image_file, default_content_type="image/jpeg")}
                    if image_file is not None
                    else {}
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateDisputeEvidenceFileResponse,
                    construct_type(
                        type_=CreateDisputeEvidenceFileResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_evidence_text(
        self,
        dispute_id: str,
        *,
        idempotency_key: str,
        evidence_text: str,
        evidence_type: typing.Optional[DisputeEvidenceType] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateDisputeEvidenceTextResponse]:
        """
        Uploads text to use as evidence for a dispute challenge.

        Parameters
        ----------
        dispute_id : str
            The ID of the dispute for which you want to upload evidence.

        idempotency_key : str
            A unique key identifying the request. For more information, see [Idempotency](https://developer.squareup.com/docs/working-with-apis/idempotency).

        evidence_text : str
            The evidence string.

        evidence_type : typing.Optional[DisputeEvidenceType]
            The type of evidence you are uploading.
            See [DisputeEvidenceType](#type-disputeevidencetype) for possible values

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateDisputeEvidenceTextResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/disputes/{jsonable_encoder(dispute_id)}/evidence-text",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "evidence_type": evidence_type,
                "evidence_text": evidence_text,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateDisputeEvidenceTextResponse,
                    construct_type(
                        type_=CreateDisputeEvidenceTextResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def submit_evidence(
        self, dispute_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[SubmitEvidenceResponse]:
        """
        Submits evidence to the cardholder's bank.

        The evidence submitted by this endpoint includes evidence uploaded
        using the [CreateDisputeEvidenceFile](api-endpoint:Disputes-CreateDisputeEvidenceFile) and
        [CreateDisputeEvidenceText](api-endpoint:Disputes-CreateDisputeEvidenceText) endpoints and
        evidence automatically provided by Square, when available. Evidence cannot be removed from
        a dispute after submission.

        Parameters
        ----------
        dispute_id : str
            The ID of the dispute for which you want to submit evidence.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SubmitEvidenceResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/disputes/{jsonable_encoder(dispute_id)}/submit-evidence",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SubmitEvidenceResponse,
                    construct_type(
                        type_=SubmitEvidenceResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from ..types.application_details_external_square_product import ApplicationDetailsExternalSquareProduct
import typing


class ApplicationDetailsParams(typing_extensions.TypedDict):
    """
    Details about the application that took the payment.
    """

    square_product: typing_extensions.NotRequired[ApplicationDetailsExternalSquareProduct]
    """
    The Square product, such as Square Point of Sale (POS), 
    Square Invoices, or Square Virtual Terminal.
    See [ExternalSquareProduct](#type-externalsquareproduct) for possible values
    """

    application_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The Square ID assigned to the application used to take the payment. 
    Application developers can use this information to identify payments that 
    their application processed. 
    For example, if a developer uses a custom application to process payments, 
    this field contains the application ID from the Developer Dashboard. 
    If a seller uses a [Square App Marketplace](https://developer.squareup.com/docs/app-marketplace) 
    application to process payments, the field contains the corresponding application ID.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class LoyaltyEventLocationFilter(UncheckedBaseModel):
    """
    Filter events by location.
    """

    location_ids: typing.List[str] = pydantic.Field()
    """
    The [location](entity:Location) IDs for loyalty events to query.
    If multiple values are specified, the endpoint uses 
    a logical OR to combine them.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .time_range import TimeRange
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class SearchEventsFilter(UncheckedBaseModel):
    """
    Criteria to filter events by.
    """

    event_types: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Filter events by event types.
    """

    merchant_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Filter events by merchant.
    """

    location_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Filter events by location.
    """

    created_at: typing.Optional[TimeRange] = pydantic.Field(default=None)
    """
    Filter events by when they were created.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .event import EventParams
from .event_metadata import EventMetadataParams


class SearchEventsResponseParams(typing_extensions.TypedDict):
    """
    Defines the fields that are included in the response body of
    a request to the [SearchEvents](api-endpoint:Events-SearchEvents) endpoint.

    Note: if there are errors processing the request, the events field will not be
    present.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Information on errors encountered during the request.
    """

    events: typing_extensions.NotRequired[typing.Sequence[EventParams]]
    """
    The list of [Event](entity:Event)s returned by the search.
    """

    metadata: typing_extensions.NotRequired[typing.Sequence[EventMetadataParams]]
    """
    Contains the metadata of an event. For more information, see [Event](entity:Event).
    """

    cursor: typing_extensions.NotRequired[str]
    """
    When a response is truncated, it includes a cursor that you can use in a subsequent request to fetch the next set of events. If empty, this is the final response.
    
    For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).
    """



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from .raw_client import RawPaymentLinksClient
from ...core.request_options import RequestOptions
from ...core.pagination import SyncPager
from ...types.payment_link import PaymentLink
from ...types.list_payment_links_response import ListPaymentLinksResponse
from ...core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...requests.quick_pay import QuickPayParams
from ...requests.order import OrderParams
from ...requests.checkout_options import CheckoutOptionsParams
from ...requests.pre_populated_data import PrePopulatedDataParams
from ...types.create_payment_link_response import CreatePaymentLinkResponse
from ...types.get_payment_link_response import GetPaymentLinkResponse
from ...requests.payment_link import PaymentLinkParams
from ...types.update_payment_link_response import UpdatePaymentLinkResponse
from ...types.delete_payment_link_response import DeletePaymentLinkResponse
from ...core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawPaymentLinksClient
from ...core.pagination import AsyncPager

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class PaymentLinksClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawPaymentLinksClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawPaymentLinksClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawPaymentLinksClient
        """
        return self._raw_client

    def list(
        self,
        *,
        cursor: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[PaymentLink]:
        """
        Lists all payment links.

        Parameters
        ----------
        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this cursor to retrieve the next set of results for the original query.
            If a cursor is not provided, the endpoint returns the first page of the results.
            For more  information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        limit : typing.Optional[int]
            A limit on the number of results to return per page. The limit is advisory and
            the implementation might return more or less results. If the supplied limit is negative, zero, or
            greater than the maximum limit of 1000, it is ignored.

            Default value: `100`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[PaymentLink]
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        response = client.checkout.payment_links.list()
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        _response = self._raw_client._client_wrapper.httpx_client.request(
            "v2/online-checkout/payment-links",
            method="GET",
            params={
                "cursor": cursor,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListPaymentLinksResponse,
                    construct_type(
                        type_=ListPaymentLinksResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    cursor=_parsed_next,
                    limit=limit,
                    request_options=request_options,
                )
                _items = _parsed_response.payment_links
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        idempotency_key: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        quick_pay: typing.Optional[QuickPayParams] = OMIT,
        order: typing.Optional[OrderParams] = OMIT,
        checkout_options: typing.Optional[CheckoutOptionsParams] = OMIT,
        pre_populated_data: typing.Optional[PrePopulatedDataParams] = OMIT,
        payment_note: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreatePaymentLinkResponse:
        """
        Creates a Square-hosted checkout page. Applications can share the resulting payment link with their buyer to pay for goods and services.

        Parameters
        ----------
        idempotency_key : typing.Optional[str]
            A unique string that identifies this `CreatePaymentLinkRequest` request.
            If you do not provide a unique string (or provide an empty string as the value),
            the endpoint treats each request as independent.

            For more information, see [Idempotency](https://developer.squareup.com/docs/working-with-apis/idempotency).

        description : typing.Optional[str]
            A description of the payment link. You provide this optional description that is useful in your
            application context. It is not used anywhere.

        quick_pay : typing.Optional[QuickPayParams]
            Describes an ad hoc item and price for which to generate a quick pay checkout link.
            For more information,
            see [Quick Pay Checkout](https://developer.squareup.com/docs/checkout-api/quick-pay-checkout).

        order : typing.Optional[OrderParams]
            Describes the `Order` for which to create a checkout link.
            For more information,
            see [Square Order Checkout](https://developer.squareup.com/docs/checkout-api/square-order-checkout).

        checkout_options : typing.Optional[CheckoutOptionsParams]
            Describes optional fields to add to the resulting checkout page.
            For more information,
            see [Optional Checkout Configurations](https://developer.squareup.com/docs/checkout-api/optional-checkout-configurations).

        pre_populated_data : typing.Optional[PrePopulatedDataParams]
            Describes fields to prepopulate in the resulting checkout page.
            For more information, see [Prepopulate the shipping address](https://developer.squareup.com/docs/checkout-api/optional-checkout-configurations#prepopulate-the-shipping-address).

        payment_note : typing.Optional[str]
            A note for the payment. After processing the payment, Square adds this note to the resulting `Payment`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreatePaymentLinkResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.checkout.payment_links.create(
            idempotency_key="cd9e25dc-d9f2-4430-aedb-61605070e95f",
            quick_pay={
                "name": "Auto Detailing",
                "price_money": {"amount": 10000, "currency": "USD"},
                "location_id": "A9Y43N9ABXZBP",
            },
        )
        """
        response = self._raw_client.create(
            idempotency_key=idempotency_key,
            description=description,
            quick_pay=quick_pay,
            order=order,
            checkout_options=checkout_options,
            pre_populated_data=pre_populated_data,
            payment_note=payment_note,
            request_options=request_options,
        )
        return response.data

    def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetPaymentLinkResponse:
        """
        Retrieves a payment link.

        Parameters
        ----------
        id : str
            The ID of link to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetPaymentLinkResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.checkout.payment_links.get(
            id="id",
        )
        """
        response = self._raw_client.get(id, request_options=request_options)
        return response.data

    def update(
        self, id: str, *, payment_link: PaymentLinkParams, request_options: typing.Optional[RequestOptions] = None
    ) -> UpdatePaymentLinkResponse:
        """
        Updates a payment link. You can update the `payment_link` fields such as
        `description`, `checkout_options`, and  `pre_populated_data`.
        You cannot update other fields such as the `order_id`, `version`, `URL`, or `timestamp` field.

        Parameters
        ----------
        id : str
            The ID of the payment link to update.

        payment_link : PaymentLinkParams
            The `payment_link` object describing the updates to apply.
            For more information, see [Update a payment link](https://developer.squareup.com/docs/checkout-api/manage-checkout#update-a-payment-link).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdatePaymentLinkResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.checkout.payment_links.update(
            id="id",
            payment_link={
                "version": 1,
                "checkout_options": {"ask_for_shipping_address": True},
            },
        )
        """
        response = self._raw_client.update(id, payment_link=payment_link, request_options=request_options)
        return response.data

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> DeletePaymentLinkResponse:
        """
        Deletes a payment link.

        Parameters
        ----------
        id : str
            The ID of the payment link to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeletePaymentLinkResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.checkout.payment_links.delete(
            id="id",
        )
        """
        response = self._raw_client.delete(id, request_options=request_options)
        return response.data


class AsyncPaymentLinksClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawPaymentLinksClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawPaymentLinksClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawPaymentLinksClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        cursor: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[PaymentLink]:
        """
        Lists all payment links.

        Parameters
        ----------
        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this cursor to retrieve the next set of results for the original query.
            If a cursor is not provided, the endpoint returns the first page of the results.
            For more  information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        limit : typing.Optional[int]
            A limit on the number of results to return per page. The limit is advisory and
            the implementation might return more or less results. If the supplied limit is negative, zero, or
            greater than the maximum limit of 1000, it is ignored.

            Default value: `100`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[PaymentLink]
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.checkout.payment_links.list()
            async for item in response:
                yield item
            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        _response = await self._raw_client._client_wrapper.httpx_client.request(
            "v2/online-checkout/payment-links",
            method="GET",
            params={
                "cursor": cursor,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListPaymentLinksResponse,
                    construct_type(
                        type_=ListPaymentLinksResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    cursor=_parsed_next,
                    limit=limit,
                    request_options=request_options,
                )
                _items = _parsed_response.payment_links
                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        idempotency_key: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        quick_pay: typing.Optional[QuickPayParams] = OMIT,
        order: typing.Optional[OrderParams] = OMIT,
        checkout_options: typing.Optional[CheckoutOptionsParams] = OMIT,
        pre_populated_data: typing.Optional[PrePopulatedDataParams] = OMIT,
        payment_note: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreatePaymentLinkResponse:
        """
        Creates a Square-hosted checkout page. Applications can share the resulting payment link with their buyer to pay for goods and services.

        Parameters
        ----------
        idempotency_key : typing.Optional[str]
            A unique string that identifies this `CreatePaymentLinkRequest` request.
            If you do not provide a unique string (or provide an empty string as the value),
            the endpoint treats each request as independent.

            For more information, see [Idempotency](https://developer.squareup.com/docs/working-with-apis/idempotency).

        description : typing.Optional[str]
            A description of the payment link. You provide this optional description that is useful in your
            application context. It is not used anywhere.

        quick_pay : typing.Optional[QuickPayParams]
            Describes an ad hoc item and price for which to generate a quick pay checkout link.
            For more information,
            see [Quick Pay Checkout](https://developer.squareup.com/docs/checkout-api/quick-pay-checkout).

        order : typing.Optional[OrderParams]
            Describes the `Order` for which to create a checkout link.
            For more information,
            see [Square Order Checkout](https://developer.squareup.com/docs/checkout-api/square-order-checkout).

        checkout_options : typing.Optional[CheckoutOptionsParams]
            Describes optional fields to add to the resulting checkout page.
            For more information,
            see [Optional Checkout Configurations](https://developer.squareup.com/docs/checkout-api/optional-checkout-configurations).

        pre_populated_data : typing.Optional[PrePopulatedDataParams]
            Describes fields to prepopulate in the resulting checkout page.
            For more information, see [Prepopulate the shipping address](https://developer.squareup.com/docs/checkout-api/optional-checkout-configurations#prepopulate-the-shipping-address).

        payment_note : typing.Optional[str]
            A note for the payment. After processing the payment, Square adds this note to the resulting `Payment`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreatePaymentLinkResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.checkout.payment_links.create(
                idempotency_key="cd9e25dc-d9f2-4430-aedb-61605070e95f",
                quick_pay={
                    "name": "Auto Detailing",
                    "price_money": {"amount": 10000, "currency": "USD"},
                    "location_id": "A9Y43N9ABXZBP",
                },
            )


        asyncio.run(main())
        """
        response = await self._raw_client.create(
            idempotency_key=idempotency_key,
            description=description,
            quick_pay=quick_pay,
            order=order,
            checkout_options=checkout_options,
            pre_populated_data=pre_populated_data,
            payment_note=payment_note,
            request_options=request_options,
        )
        return response.data

    async def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetPaymentLinkResponse:
        """
        Retrieves a payment link.

        Parameters
        ----------
        id : str
            The ID of link to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetPaymentLinkResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.checkout.payment_links.get(
                id="id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(id, request_options=request_options)
        return response.data

    async def update(
        self, id: str, *, payment_link: PaymentLinkParams, request_options: typing.Optional[RequestOptions] = None
    ) -> UpdatePaymentLinkResponse:
        """
        Updates a payment link. You can update the `payment_link` fields such as
        `description`, `checkout_options`, and  `pre_populated_data`.
        You cannot update other fields such as the `order_id`, `version`, `URL`, or `timestamp` field.

        Parameters
        ----------
        id : str
            The ID of the payment link to update.

        payment_link : PaymentLinkParams
            The `payment_link` object describing the updates to apply.
            For more information, see [Update a payment link](https://developer.squareup.com/docs/checkout-api/manage-checkout#update-a-payment-link).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdatePaymentLinkResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.checkout.payment_links.update(
                id="id",
                payment_link={
                    "version": 1,
                    "checkout_options": {"ask_for_shipping_address": True},
                },
            )


        asyncio.run(main())
        """
        response = await self._raw_client.update(id, payment_link=payment_link, request_options=request_options)
        return response.data

    async def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeletePaymentLinkResponse:
        """
        Deletes a payment link.

        Parameters
        ----------
        id : str
            The ID of the payment link to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeletePaymentLinkResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.checkout.payment_links.delete(
                id="id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.delete(id, request_options=request_options)
        return response.data



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .money import Money
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class OrderRoundingAdjustment(UncheckedBaseModel):
    """
    A rounding adjustment of the money being returned. Commonly used to apply cash rounding
    when the minimum unit of the account is smaller than the lowest physical denomination of the currency.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    A unique ID that identifies the rounding adjustment only within this order.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the rounding adjustment from the original sale order.
    """

    amount_money: typing.Optional[Money] = pydantic.Field(default=None)
    """
    The actual rounding adjustment amount.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing
from ..types.loyalty_event_type import LoyaltyEventType


class LoyaltyEventTypeFilterParams(typing_extensions.TypedDict):
    """
    Filter events by event type.
    """

    types: typing.Sequence[LoyaltyEventType]
    """
    The loyalty event types used to filter the result.
    If multiple values are specified, the endpoint uses a 
    logical OR to combine them.
    See [LoyaltyEventType](#type-loyaltyeventtype) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from .raw_client import RawOAuthClient
from ..core.request_options import RequestOptions
from ..types.revoke_token_response import RevokeTokenResponse
from ..types.obtain_token_response import ObtainTokenResponse
from ..types.retrieve_token_status_response import RetrieveTokenStatusResponse
from ..core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawOAuthClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class OAuthClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawOAuthClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawOAuthClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawOAuthClient
        """
        return self._raw_client

    def revoke_token(
        self,
        *,
        client_id: typing.Optional[str] = OMIT,
        access_token: typing.Optional[str] = OMIT,
        merchant_id: typing.Optional[str] = OMIT,
        revoke_only_access_token: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RevokeTokenResponse:
        """
        Revokes an access token generated with the OAuth flow.

        If an account has more than one OAuth access token for your application, this
        endpoint revokes all of them, regardless of which token you specify.

        __Important:__ The `Authorization` header for this endpoint must have the
        following format:

        ```
        Authorization: Client APPLICATION_SECRET
        ```

        Replace `APPLICATION_SECRET` with the application secret on the **OAuth**
        page for your application in the Developer Dashboard.

        Parameters
        ----------
        client_id : typing.Optional[str]
            The Square-issued ID for your application, which is available on the **OAuth** page in the
            [Developer Dashboard](https://developer.squareup.com/apps).

        access_token : typing.Optional[str]
            The access token of the merchant whose token you want to revoke.
            Do not provide a value for `merchant_id` if you provide this parameter.

        merchant_id : typing.Optional[str]
            The ID of the merchant whose token you want to revoke.
            Do not provide a value for `access_token` if you provide this parameter.

        revoke_only_access_token : typing.Optional[bool]
            If `true`, terminate the given single access token, but do not
            terminate the entire authorization.
            Default: `false`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RevokeTokenResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.o_auth.revoke_token(
            client_id="CLIENT_ID",
            access_token="ACCESS_TOKEN",
        )
        """
        response = self._raw_client.revoke_token(
            client_id=client_id,
            access_token=access_token,
            merchant_id=merchant_id,
            revoke_only_access_token=revoke_only_access_token,
            request_options=request_options,
        )
        return response.data

    def obtain_token(
        self,
        *,
        client_id: str,
        grant_type: str,
        client_secret: typing.Optional[str] = OMIT,
        code: typing.Optional[str] = OMIT,
        redirect_uri: typing.Optional[str] = OMIT,
        refresh_token: typing.Optional[str] = OMIT,
        migration_token: typing.Optional[str] = OMIT,
        scopes: typing.Optional[typing.Sequence[str]] = OMIT,
        short_lived: typing.Optional[bool] = OMIT,
        code_verifier: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ObtainTokenResponse:
        """
        Returns an OAuth access token and refresh token using the `authorization_code`
        or `refresh_token` grant type.

        When `grant_type` is `authorization_code`:
        - With the [code flow](https://developer.squareup.com/docs/oauth-api/overview#code-flow),
        provide `code`, `client_id`, and `client_secret`.
        - With the [PKCE flow](https://developer.squareup.com/docs/oauth-api/overview#pkce-flow),
        provide `code`, `client_id`, and `code_verifier`.

        When `grant_type` is `refresh_token`:
        - With the code flow, provide `refresh_token`, `client_id`, and `client_secret`.
        The response returns the same refresh token provided in the request.
        - With the PKCE flow, provide `refresh_token` and `client_id`. The response returns
        a new refresh token.

        You can use the `scopes` parameter to limit the set of permissions authorized by the
        access token. You can use the `short_lived` parameter to create an access token that
        expires in 24 hours.

        __Important:__ OAuth tokens should be encrypted and stored on a secure server.
        Application clients should never interact directly with OAuth tokens.

        Parameters
        ----------
        client_id : str
            The Square-issued ID of your application, which is available as the **Application ID**
            on the **OAuth** page in the [Developer Console](https://developer.squareup.com/apps).

            Required for the code flow and PKCE flow for any grant type.

        grant_type : str
            The method used to obtain an OAuth access token. The request must include the
            credential that corresponds to the specified grant type. Valid values are:
            - `authorization_code` - Requires the `code` field.
            - `refresh_token` - Requires the `refresh_token` field.
            - `migration_token` - LEGACY for access tokens obtained using a Square API version prior
            to 2019-03-13. Requires the `migration_token` field.

        client_secret : typing.Optional[str]
            The secret key for your application, which is available as the **Application secret**
            on the **OAuth** page in the [Developer Console](https://developer.squareup.com/apps).

            Required for the code flow for any grant type. Don't confuse your client secret with your
            personal access token.

        code : typing.Optional[str]
            The authorization code to exchange for an OAuth access token. This is the `code`
            value that Square sent to your redirect URL in the authorization response.

            Required for the code flow and PKCE flow if `grant_type` is `authorization_code`.

        redirect_uri : typing.Optional[str]
            The redirect URL for your application, which you registered as the **Redirect URL**
            on the **OAuth** page in the [Developer Console](https://developer.squareup.com/apps).

            Required for the code flow and PKCE flow if `grant_type` is `authorization_code` and
            you provided the `redirect_uri` parameter in your authorization URL.

        refresh_token : typing.Optional[str]
            A valid refresh token used to generate a new OAuth access token. This is a
            refresh token that was returned in a previous `ObtainToken` response.

            Required for the code flow and PKCE flow if `grant_type` is `refresh_token`.

        migration_token : typing.Optional[str]
            __LEGACY__ A valid access token (obtained using a Square API version prior to 2019-03-13)
            used to generate a new OAuth access token.

            Required if `grant_type` is `migration_token`. For more information, see
            [Migrate to Using Refresh Tokens](https://developer.squareup.com/docs/oauth-api/migrate-to-refresh-tokens).

        scopes : typing.Optional[typing.Sequence[str]]
            The list of permissions that are explicitly requested for the access token.
            For example, ["MERCHANT_PROFILE_READ","PAYMENTS_READ","BANK_ACCOUNTS_READ"].

            The returned access token is limited to the permissions that are the intersection
            of these requested permissions and those authorized by the provided `refresh_token`.

            Optional for the code flow and PKCE flow if `grant_type` is `refresh_token`.

        short_lived : typing.Optional[bool]
            Indicates whether the returned access token should expire in 24 hours.

            Optional for the code flow and PKCE flow for any grant type. The default value is `false`.

        code_verifier : typing.Optional[str]
            The secret your application generated for the authorization request used to
            obtain the authorization code. This is the source of the `code_challenge` hash you
            provided in your authorization URL.

            Required for the PKCE flow if `grant_type` is `authorization_code`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ObtainTokenResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.o_auth.obtain_token(
            client_id="sq0idp-uaPHILoPzWZk3tlJqlML0g",
            client_secret="sq0csp-30a-4C_tVOnTh14Piza2BfTPBXyLafLPWSzY1qAjeBfM",
            code="sq0cgb-l0SBqxs4uwxErTVyYOdemg",
            grant_type="authorization_code",
        )
        """
        response = self._raw_client.obtain_token(
            client_id=client_id,
            grant_type=grant_type,
            client_secret=client_secret,
            code=code,
            redirect_uri=redirect_uri,
            refresh_token=refresh_token,
            migration_token=migration_token,
            scopes=scopes,
            short_lived=short_lived,
            code_verifier=code_verifier,
            request_options=request_options,
        )
        return response.data

    def retrieve_token_status(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> RetrieveTokenStatusResponse:
        """
        Returns information about an [OAuth access token](https://developer.squareup.com/docs/build-basics/access-tokens#get-an-oauth-access-token) or an application’s [personal access token](https://developer.squareup.com/docs/build-basics/access-tokens#get-a-personal-access-token).

        Add the access token to the Authorization header of the request.

        __Important:__ The `Authorization` header you provide to this endpoint must have the following format:

        ```
        Authorization: Bearer ACCESS_TOKEN
        ```

        where `ACCESS_TOKEN` is a
        [valid production authorization credential](https://developer.squareup.com/docs/build-basics/access-tokens).

        If the access token is expired or not a valid access token, the endpoint returns an `UNAUTHORIZED` error.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RetrieveTokenStatusResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.o_auth.retrieve_token_status()
        """
        response = self._raw_client.retrieve_token_status(request_options=request_options)
        return response.data

    def authorize(self, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.o_auth.authorize()
        """
        response = self._raw_client.authorize(request_options=request_options)
        return response.data


class AsyncOAuthClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawOAuthClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawOAuthClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawOAuthClient
        """
        return self._raw_client

    async def revoke_token(
        self,
        *,
        client_id: typing.Optional[str] = OMIT,
        access_token: typing.Optional[str] = OMIT,
        merchant_id: typing.Optional[str] = OMIT,
        revoke_only_access_token: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RevokeTokenResponse:
        """
        Revokes an access token generated with the OAuth flow.

        If an account has more than one OAuth access token for your application, this
        endpoint revokes all of them, regardless of which token you specify.

        __Important:__ The `Authorization` header for this endpoint must have the
        following format:

        ```
        Authorization: Client APPLICATION_SECRET
        ```

        Replace `APPLICATION_SECRET` with the application secret on the **OAuth**
        page for your application in the Developer Dashboard.

        Parameters
        ----------
        client_id : typing.Optional[str]
            The Square-issued ID for your application, which is available on the **OAuth** page in the
            [Developer Dashboard](https://developer.squareup.com/apps).

        access_token : typing.Optional[str]
            The access token of the merchant whose token you want to revoke.
            Do not provide a value for `merchant_id` if you provide this parameter.

        merchant_id : typing.Optional[str]
            The ID of the merchant whose token you want to revoke.
            Do not provide a value for `access_token` if you provide this parameter.

        revoke_only_access_token : typing.Optional[bool]
            If `true`, terminate the given single access token, but do not
            terminate the entire authorization.
            Default: `false`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RevokeTokenResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.o_auth.revoke_token(
                client_id="CLIENT_ID",
                access_token="ACCESS_TOKEN",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.revoke_token(
            client_id=client_id,
            access_token=access_token,
            merchant_id=merchant_id,
            revoke_only_access_token=revoke_only_access_token,
            request_options=request_options,
        )
        return response.data

    async def obtain_token(
        self,
        *,
        client_id: str,
        grant_type: str,
        client_secret: typing.Optional[str] = OMIT,
        code: typing.Optional[str] = OMIT,
        redirect_uri: typing.Optional[str] = OMIT,
        refresh_token: typing.Optional[str] = OMIT,
        migration_token: typing.Optional[str] = OMIT,
        scopes: typing.Optional[typing.Sequence[str]] = OMIT,
        short_lived: typing.Optional[bool] = OMIT,
        code_verifier: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ObtainTokenResponse:
        """
        Returns an OAuth access token and refresh token using the `authorization_code`
        or `refresh_token` grant type.

        When `grant_type` is `authorization_code`:
        - With the [code flow](https://developer.squareup.com/docs/oauth-api/overview#code-flow),
        provide `code`, `client_id`, and `client_secret`.
        - With the [PKCE flow](https://developer.squareup.com/docs/oauth-api/overview#pkce-flow),
        provide `code`, `client_id`, and `code_verifier`.

        When `grant_type` is `refresh_token`:
        - With the code flow, provide `refresh_token`, `client_id`, and `client_secret`.
        The response returns the same refresh token provided in the request.
        - With the PKCE flow, provide `refresh_token` and `client_id`. The response returns
        a new refresh token.

        You can use the `scopes` parameter to limit the set of permissions authorized by the
        access token. You can use the `short_lived` parameter to create an access token that
        expires in 24 hours.

        __Important:__ OAuth tokens should be encrypted and stored on a secure server.
        Application clients should never interact directly with OAuth tokens.

        Parameters
        ----------
        client_id : str
            The Square-issued ID of your application, which is available as the **Application ID**
            on the **OAuth** page in the [Developer Console](https://developer.squareup.com/apps).

            Required for the code flow and PKCE flow for any grant type.

        grant_type : str
            The method used to obtain an OAuth access token. The request must include the
            credential that corresponds to the specified grant type. Valid values are:
            - `authorization_code` - Requires the `code` field.
            - `refresh_token` - Requires the `refresh_token` field.
            - `migration_token` - LEGACY for access tokens obtained using a Square API version prior
            to 2019-03-13. Requires the `migration_token` field.

        client_secret : typing.Optional[str]
            The secret key for your application, which is available as the **Application secret**
            on the **OAuth** page in the [Developer Console](https://developer.squareup.com/apps).

            Required for the code flow for any grant type. Don't confuse your client secret with your
            personal access token.

        code : typing.Optional[str]
            The authorization code to exchange for an OAuth access token. This is the `code`
            value that Square sent to your redirect URL in the authorization response.

            Required for the code flow and PKCE flow if `grant_type` is `authorization_code`.

        redirect_uri : typing.Optional[str]
            The redirect URL for your application, which you registered as the **Redirect URL**
            on the **OAuth** page in the [Developer Console](https://developer.squareup.com/apps).

            Required for the code flow and PKCE flow if `grant_type` is `authorization_code` and
            you provided the `redirect_uri` parameter in your authorization URL.

        refresh_token : typing.Optional[str]
            A valid refresh token used to generate a new OAuth access token. This is a
            refresh token that was returned in a previous `ObtainToken` response.

            Required for the code flow and PKCE flow if `grant_type` is `refresh_token`.

        migration_token : typing.Optional[str]
            __LEGACY__ A valid access token (obtained using a Square API version prior to 2019-03-13)
            used to generate a new OAuth access token.

            Required if `grant_type` is `migration_token`. For more information, see
            [Migrate to Using Refresh Tokens](https://developer.squareup.com/docs/oauth-api/migrate-to-refresh-tokens).

        scopes : typing.Optional[typing.Sequence[str]]
            The list of permissions that are explicitly requested for the access token.
            For example, ["MERCHANT_PROFILE_READ","PAYMENTS_READ","BANK_ACCOUNTS_READ"].

            The returned access token is limited to the permissions that are the intersection
            of these requested permissions and those authorized by the provided `refresh_token`.

            Optional for the code flow and PKCE flow if `grant_type` is `refresh_token`.

        short_lived : typing.Optional[bool]
            Indicates whether the returned access token should expire in 24 hours.

            Optional for the code flow and PKCE flow for any grant type. The default value is `false`.

        code_verifier : typing.Optional[str]
            The secret your application generated for the authorization request used to
            obtain the authorization code. This is the source of the `code_challenge` hash you
            provided in your authorization URL.

            Required for the PKCE flow if `grant_type` is `authorization_code`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ObtainTokenResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.o_auth.obtain_token(
                client_id="sq0idp-uaPHILoPzWZk3tlJqlML0g",
                client_secret="sq0csp-30a-4C_tVOnTh14Piza2BfTPBXyLafLPWSzY1qAjeBfM",
                code="sq0cgb-l0SBqxs4uwxErTVyYOdemg",
                grant_type="authorization_code",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.obtain_token(
            client_id=client_id,
            grant_type=grant_type,
            client_secret=client_secret,
            code=code,
            redirect_uri=redirect_uri,
            refresh_token=refresh_token,
            migration_token=migration_token,
            scopes=scopes,
            short_lived=short_lived,
            code_verifier=code_verifier,
            request_options=request_options,
        )
        return response.data

    async def retrieve_token_status(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> RetrieveTokenStatusResponse:
        """
        Returns information about an [OAuth access token](https://developer.squareup.com/docs/build-basics/access-tokens#get-an-oauth-access-token) or an application’s [personal access token](https://developer.squareup.com/docs/build-basics/access-tokens#get-a-personal-access-token).

        Add the access token to the Authorization header of the request.

        __Important:__ The `Authorization` header you provide to this endpoint must have the following format:

        ```
        Authorization: Bearer ACCESS_TOKEN
        ```

        where `ACCESS_TOKEN` is a
        [valid production authorization credential](https://developer.squareup.com/docs/build-basics/access-tokens).

        If the access token is expired or not a valid access token, the endpoint returns an `UNAUTHORIZED` error.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RetrieveTokenStatusResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.o_auth.retrieve_token_status()


        asyncio.run(main())
        """
        response = await self._raw_client.retrieve_token_status(request_options=request_options)
        return response.data

    async def authorize(self, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.o_auth.authorize()


        asyncio.run(main())
        """
        response = await self._raw_client.authorize(request_options=request_options)
        return response.data



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .error import Error
import pydantic
from .inventory_change import InventoryChange
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class BatchGetInventoryChangesResponse(UncheckedBaseModel):
    errors: typing.Optional[typing.List[Error]] = pydantic.Field(default=None)
    """
    Any errors that occurred during the request.
    """

    changes: typing.Optional[typing.List[InventoryChange]] = pydantic.Field(default=None)
    """
    The current calculated inventory changes for the requested objects
    and locations.
    """

    cursor: typing.Optional[str] = pydantic.Field(default=None)
    """
    The pagination cursor to be used in a subsequent request. If unset,
    this is the final response.
    See the [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination) guide for more information.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CustomerTextFilter(UncheckedBaseModel):
    """
    A filter to select customers based on exact or fuzzy matching of
    customer attributes against a specified query. Depending on the customer attributes,
    the filter can be case-sensitive. This filter can be exact or fuzzy, but it cannot be both.
    """

    exact: typing.Optional[str] = pydantic.Field(default=None)
    """
    Use the exact filter to select customers whose attributes match exactly the specified query.
    """

    fuzzy: typing.Optional[str] = pydantic.Field(default=None)
    """
    Use the fuzzy filter to select customers whose attributes match the specified query 
    in a fuzzy manner. When the fuzzy option is used, search queries are tokenized, and then 
    each query token must be matched somewhere in the searched attribute. For single token queries, 
    this is effectively the same behavior as a partial match operation.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.client_wrapper import SyncClientWrapper
from .raw_client import RawCashDrawersClient
from .shifts.client import ShiftsClient
from ..core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawCashDrawersClient
from .shifts.client import AsyncShiftsClient


class CashDrawersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawCashDrawersClient(client_wrapper=client_wrapper)
        self.shifts = ShiftsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawCashDrawersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawCashDrawersClient
        """
        return self._raw_client


class AsyncCashDrawersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawCashDrawersClient(client_wrapper=client_wrapper)
        self.shifts = AsyncShiftsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawCashDrawersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawCashDrawersClient
        """
        return self._raw_client



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Job(UncheckedBaseModel):
    """
    Represents a job that can be assigned to [team members](entity:TeamMember). This object defines the
    job's title and tip eligibility. Compensation is defined in a [job assignment](entity:JobAssignment)
    in a team member's wage setting.
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    **Read only** The unique Square-assigned ID of the job. If you need a job ID for an API request,
    call [ListJobs](api-endpoint:Team-ListJobs) or use the ID returned when you created the job.
    You can also get job IDs from a team member's wage setting.
    """

    title: typing.Optional[str] = pydantic.Field(default=None)
    """
    The title of the job.
    """

    is_tip_eligible: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates whether team members can earn tips for the job.
    """

    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp when the job was created, in RFC 3339 format.
    """

    updated_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp when the job was last updated, in RFC 3339 format.
    """

    version: typing.Optional[int] = pydantic.Field(default=None)
    """
    **Read only** The current version of the job. Include this field in `UpdateJob` requests to enable
    [optimistic concurrency](https://developer.squareup.com/docs/working-with-apis/optimistic-concurrency)
    control and avoid overwrites from concurrent requests. Requests fail if the provided version doesn't
    match the server version at the time of the request.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

BankAccountType = typing.Union[
    typing.Literal["CHECKING", "SAVINGS", "INVESTMENT", "OTHER", "BUSINESS_CHECKING"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..types.loyalty_promotion_incentive_type import LoyaltyPromotionIncentiveType
import typing_extensions
from .loyalty_promotion_incentive_points_multiplier_data import LoyaltyPromotionIncentivePointsMultiplierDataParams
from .loyalty_promotion_incentive_points_addition_data import LoyaltyPromotionIncentivePointsAdditionDataParams


class LoyaltyPromotionIncentiveParams(typing_extensions.TypedDict):
    """
    Represents how points for a [loyalty promotion](entity:LoyaltyPromotion) are calculated,
    either by multiplying the points earned from the base program or by adding a specified number
    of points to the points earned from the base program.
    """

    type: LoyaltyPromotionIncentiveType
    """
    The type of points incentive.
    See [LoyaltyPromotionIncentiveType](#type-loyaltypromotionincentivetype) for possible values
    """

    points_multiplier_data: typing_extensions.NotRequired[LoyaltyPromotionIncentivePointsMultiplierDataParams]
    """
    Additional data for a `POINTS_MULTIPLIER` incentive type.
    """

    points_addition_data: typing_extensions.NotRequired[LoyaltyPromotionIncentivePointsAdditionDataParams]
    """
    Additional data for a `POINTS_ADDITION` incentive type.
    """



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from .raw_client import RawOrdersClient
from .custom_attribute_definitions.client import CustomAttributeDefinitionsClient
from .custom_attributes.client import CustomAttributesClient
from ..requests.order import OrderParams
from ..core.request_options import RequestOptions
from ..types.create_order_response import CreateOrderResponse
from ..types.batch_get_orders_response import BatchGetOrdersResponse
from ..requests.order_reward import OrderRewardParams
from ..types.calculate_order_response import CalculateOrderResponse
from ..types.clone_order_response import CloneOrderResponse
from ..requests.search_orders_query import SearchOrdersQueryParams
from ..types.search_orders_response import SearchOrdersResponse
from ..types.get_order_response import GetOrderResponse
from ..types.update_order_response import UpdateOrderResponse
from ..types.pay_order_response import PayOrderResponse
from ..core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawOrdersClient
from .custom_attribute_definitions.client import AsyncCustomAttributeDefinitionsClient
from .custom_attributes.client import AsyncCustomAttributesClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class OrdersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawOrdersClient(client_wrapper=client_wrapper)
        self.custom_attribute_definitions = CustomAttributeDefinitionsClient(client_wrapper=client_wrapper)

        self.custom_attributes = CustomAttributesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawOrdersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawOrdersClient
        """
        return self._raw_client

    def create(
        self,
        *,
        order: typing.Optional[OrderParams] = OMIT,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateOrderResponse:
        """
        Creates a new [order](entity:Order) that can include information about products for
        purchase and settings to apply to the purchase.

        To pay for a created order, see
        [Pay for Orders](https://developer.squareup.com/docs/orders-api/pay-for-orders).

        You can modify open orders using the [UpdateOrder](api-endpoint:Orders-UpdateOrder) endpoint.

        Parameters
        ----------
        order : typing.Optional[OrderParams]
            The order to create. If this field is set, the only other top-level field that can be
            set is the `idempotency_key`.

        idempotency_key : typing.Optional[str]
            A value you specify that uniquely identifies this
            order among orders you have created.

            If you are unsure whether a particular order was created successfully,
            you can try it again with the same idempotency key without
            worrying about creating duplicate orders.

            For more information, see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateOrderResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.orders.create(
            order={
                "location_id": "057P5VYJ4A5X1",
                "reference_id": "my-order-001",
                "line_items": [
                    {
                        "name": "New York Strip Steak",
                        "quantity": "1",
                        "base_price_money": {"amount": 1599, "currency": "USD"},
                    },
                    {
                        "quantity": "2",
                        "catalog_object_id": "BEMYCSMIJL46OCDV4KYIKXIB",
                        "modifiers": [
                            {"catalog_object_id": "CHQX7Y4KY6N5KINJKZCFURPZ"}
                        ],
                        "applied_discounts": [{"discount_uid": "one-dollar-off"}],
                    },
                ],
                "taxes": [
                    {
                        "uid": "state-sales-tax",
                        "name": "State Sales Tax",
                        "percentage": "9",
                        "scope": "ORDER",
                    }
                ],
                "discounts": [
                    {
                        "uid": "labor-day-sale",
                        "name": "Labor Day Sale",
                        "percentage": "5",
                        "scope": "ORDER",
                    },
                    {
                        "uid": "membership-discount",
                        "catalog_object_id": "DB7L55ZH2BGWI4H23ULIWOQ7",
                        "scope": "ORDER",
                    },
                    {
                        "uid": "one-dollar-off",
                        "name": "Sale - $1.00 off",
                        "amount_money": {"amount": 100, "currency": "USD"},
                        "scope": "LINE_ITEM",
                    },
                ],
            },
            idempotency_key="8193148c-9586-11e6-99f9-28cfe92138cf",
        )
        """
        response = self._raw_client.create(
            order=order, idempotency_key=idempotency_key, request_options=request_options
        )
        return response.data

    def batch_get(
        self,
        *,
        order_ids: typing.Sequence[str],
        location_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BatchGetOrdersResponse:
        """
        Retrieves a set of [orders](entity:Order) by their IDs.

        If a given order ID does not exist, the ID is ignored instead of generating an error.

        Parameters
        ----------
        order_ids : typing.Sequence[str]
            The IDs of the orders to retrieve. A maximum of 100 orders can be retrieved per request.

        location_id : typing.Optional[str]
            The ID of the location for these orders. This field is optional: omit it to retrieve
            orders within the scope of the current authorization's merchant ID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BatchGetOrdersResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.orders.batch_get(
            location_id="057P5VYJ4A5X1",
            order_ids=["CAISEM82RcpmcFBM0TfOyiHV3es", "CAISENgvlJ6jLWAzERDzjyHVybY"],
        )
        """
        response = self._raw_client.batch_get(
            order_ids=order_ids, location_id=location_id, request_options=request_options
        )
        return response.data

    def calculate(
        self,
        *,
        order: OrderParams,
        proposed_rewards: typing.Optional[typing.Sequence[OrderRewardParams]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CalculateOrderResponse:
        """
        Enables applications to preview order pricing without creating an order.

        Parameters
        ----------
        order : OrderParams
            The order to be calculated. Expects the entire order, not a sparse update.

        proposed_rewards : typing.Optional[typing.Sequence[OrderRewardParams]]
            Identifies one or more loyalty reward tiers to apply during the order calculation.
            The discounts defined by the reward tiers are added to the order only to preview the
            effect of applying the specified rewards. The rewards do not correspond to actual
            redemptions; that is, no `reward`s are created. Therefore, the reward `id`s are
            random strings used only to reference the reward tier.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CalculateOrderResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.orders.calculate(
            order={
                "location_id": "D7AVYMEAPJ3A3",
                "line_items": [
                    {
                        "name": "Item 1",
                        "quantity": "1",
                        "base_price_money": {"amount": 500, "currency": "USD"},
                    },
                    {
                        "name": "Item 2",
                        "quantity": "2",
                        "base_price_money": {"amount": 300, "currency": "USD"},
                    },
                ],
                "discounts": [
                    {"name": "50% Off", "percentage": "50", "scope": "ORDER"}
                ],
            },
        )
        """
        response = self._raw_client.calculate(
            order=order, proposed_rewards=proposed_rewards, request_options=request_options
        )
        return response.data

    def clone(
        self,
        *,
        order_id: str,
        version: typing.Optional[int] = OMIT,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CloneOrderResponse:
        """
        Creates a new order, in the `DRAFT` state, by duplicating an existing order. The newly created order has
        only the core fields (such as line items, taxes, and discounts) copied from the original order.

        Parameters
        ----------
        order_id : str
            The ID of the order to clone.

        version : typing.Optional[int]
            An optional order version for concurrency protection.

            If a version is provided, it must match the latest stored version of the order to clone.
            If a version is not provided, the API clones the latest version.

        idempotency_key : typing.Optional[str]
            A value you specify that uniquely identifies this clone request.

            If you are unsure whether a particular order was cloned successfully,
            you can reattempt the call with the same idempotency key without
            worrying about creating duplicate cloned orders.
            The originally cloned order is returned.

            For more information, see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CloneOrderResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.orders.clone(
            order_id="ZAISEM52YcpmcWAzERDOyiWS123",
            version=3,
            idempotency_key="UNIQUE_STRING",
        )
        """
        response = self._raw_client.clone(
            order_id=order_id, version=version, idempotency_key=idempotency_key, request_options=request_options
        )
        return response.data

    def search(
        self,
        *,
        location_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        query: typing.Optional[SearchOrdersQueryParams] = OMIT,
        limit: typing.Optional[int] = OMIT,
        return_entries: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SearchOrdersResponse:
        """
        Search all orders for one or more locations. Orders include all sales,
        returns, and exchanges regardless of how or when they entered the Square
        ecosystem (such as Point of Sale, Invoices, and Connect APIs).

        `SearchOrders` requests need to specify which locations to search and define a
        [SearchOrdersQuery](entity:SearchOrdersQuery) object that controls
        how to sort or filter the results. Your `SearchOrdersQuery` can:

          Set filter criteria.
          Set the sort order.
          Determine whether to return results as complete `Order` objects or as
        [OrderEntry](entity:OrderEntry) objects.

        Note that details for orders processed with Square Point of Sale while in
        offline mode might not be transmitted to Square for up to 72 hours. Offline
        orders have a `created_at` value that reflects the time the order was created,
        not the time it was subsequently transmitted to Square.

        Parameters
        ----------
        location_ids : typing.Optional[typing.Sequence[str]]
            The location IDs for the orders to query. All locations must belong to
            the same merchant.

            Max: 10 location IDs.

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this cursor to retrieve the next set of results for your original query.
            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        query : typing.Optional[SearchOrdersQueryParams]
            Query conditions used to filter or sort the results. Note that when
            retrieving additional pages using a cursor, you must use the original query.

        limit : typing.Optional[int]
            The maximum number of results to be returned in a single page.

            Default: `500`
            Max: `1000`

        return_entries : typing.Optional[bool]
            A Boolean that controls the format of the search results. If `true`,
            `SearchOrders` returns [OrderEntry](entity:OrderEntry) objects. If `false`, `SearchOrders`
            returns complete order objects.

            Default: `false`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SearchOrdersResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.orders.search(
            location_ids=["057P5VYJ4A5X1", "18YC4JDH91E1H"],
            query={
                "filter": {
                    "state_filter": {"states": ["COMPLETED"]},
                    "date_time_filter": {
                        "closed_at": {
                            "start_at": "2018-03-03T20:00:00+00:00",
                            "end_at": "2019-03-04T21:54:45+00:00",
                        }
                    },
                },
                "sort": {"sort_field": "CLOSED_AT", "sort_order": "DESC"},
            },
            limit=3,
            return_entries=True,
        )
        """
        response = self._raw_client.search(
            location_ids=location_ids,
            cursor=cursor,
            query=query,
            limit=limit,
            return_entries=return_entries,
            request_options=request_options,
        )
        return response.data

    def get(self, order_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetOrderResponse:
        """
        Retrieves an [Order](entity:Order) by ID.

        Parameters
        ----------
        order_id : str
            The ID of the order to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetOrderResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.orders.get(
            order_id="order_id",
        )
        """
        response = self._raw_client.get(order_id, request_options=request_options)
        return response.data

    def update(
        self,
        order_id: str,
        *,
        order: typing.Optional[OrderParams] = OMIT,
        fields_to_clear: typing.Optional[typing.Sequence[str]] = OMIT,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateOrderResponse:
        """
        Updates an open [order](entity:Order) by adding, replacing, or deleting
        fields. Orders with a `COMPLETED` or `CANCELED` state cannot be updated.

        An `UpdateOrder` request requires the following:

        - The `order_id` in the endpoint path, identifying the order to update.
        - The latest `version` of the order to update.
        - The [sparse order](https://developer.squareup.com/docs/orders-api/manage-orders/update-orders#sparse-order-objects)
        containing only the fields to update and the version to which the update is
        being applied.
        - If deleting fields, the [dot notation paths](https://developer.squareup.com/docs/orders-api/manage-orders/update-orders#identifying-fields-to-delete)
        identifying the fields to clear.

        To pay for an order, see
        [Pay for Orders](https://developer.squareup.com/docs/orders-api/pay-for-orders).

        Parameters
        ----------
        order_id : str
            The ID of the order to update.

        order : typing.Optional[OrderParams]
            The [sparse order](https://developer.squareup.com/docs/orders-api/manage-orders/update-orders#sparse-order-objects)
            containing only the fields to update and the version to which the update is
            being applied.

        fields_to_clear : typing.Optional[typing.Sequence[str]]
            The [dot notation paths](https://developer.squareup.com/docs/orders-api/manage-orders/update-orders#identifying-fields-to-delete)
            fields to clear. For example, `line_items[uid].note`.
            For more information, see [Deleting fields](https://developer.squareup.com/docs/orders-api/manage-orders/update-orders#deleting-fields).

        idempotency_key : typing.Optional[str]
            A value you specify that uniquely identifies this update request.

            If you are unsure whether a particular update was applied to an order successfully,
            you can reattempt it with the same idempotency key without
            worrying about creating duplicate updates to the order.
            The latest order version is returned.

            For more information, see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateOrderResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.orders.update(
            order_id="order_id",
            order={
                "location_id": "location_id",
                "line_items": [
                    {
                        "uid": "cookie_uid",
                        "name": "COOKIE",
                        "quantity": "2",
                        "base_price_money": {"amount": 200, "currency": "USD"},
                    }
                ],
                "version": 1,
            },
            fields_to_clear=["discounts"],
            idempotency_key="UNIQUE_STRING",
        )
        """
        response = self._raw_client.update(
            order_id,
            order=order,
            fields_to_clear=fields_to_clear,
            idempotency_key=idempotency_key,
            request_options=request_options,
        )
        return response.data

    def pay(
        self,
        order_id: str,
        *,
        idempotency_key: str,
        order_version: typing.Optional[int] = OMIT,
        payment_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayOrderResponse:
        """
        Pay for an [order](entity:Order) using one or more approved [payments](entity:Payment)
        or settle an order with a total of `0`.

        The total of the `payment_ids` listed in the request must be equal to the order
        total. Orders with a total amount of `0` can be marked as paid by specifying an empty
        array of `payment_ids` in the request.

        To be used with `PayOrder`, a payment must:

        - Reference the order by specifying the `order_id` when [creating the payment](api-endpoint:Payments-CreatePayment).
        Any approved payments that reference the same `order_id` not specified in the
        `payment_ids` is canceled.
        - Be approved with [delayed capture](https://developer.squareup.com/docs/payments-api/take-payments/card-payments/delayed-capture).
        Using a delayed capture payment with `PayOrder` completes the approved payment.

        Parameters
        ----------
        order_id : str
            The ID of the order being paid.

        idempotency_key : str
            A value you specify that uniquely identifies this request among requests you have sent. If
            you are unsure whether a particular payment request was completed successfully, you can reattempt
            it with the same idempotency key without worrying about duplicate payments.

            For more information, see [Idempotency](https://developer.squareup.com/docs/working-with-apis/idempotency).

        order_version : typing.Optional[int]
            The version of the order being paid. If not supplied, the latest version will be paid.

        payment_ids : typing.Optional[typing.Sequence[str]]
            The IDs of the [payments](entity:Payment) to collect.
            The payment total must match the order total.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayOrderResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.orders.pay(
            order_id="order_id",
            idempotency_key="c043a359-7ad9-4136-82a9-c3f1d66dcbff",
            payment_ids=["EnZdNAlWCmfh6Mt5FMNST1o7taB", "0LRiVlbXVwe8ozu4KbZxd12mvaB"],
        )
        """
        response = self._raw_client.pay(
            order_id,
            idempotency_key=idempotency_key,
            order_version=order_version,
            payment_ids=payment_ids,
            request_options=request_options,
        )
        return response.data


class AsyncOrdersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawOrdersClient(client_wrapper=client_wrapper)
        self.custom_attribute_definitions = AsyncCustomAttributeDefinitionsClient(client_wrapper=client_wrapper)

        self.custom_attributes = AsyncCustomAttributesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawOrdersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawOrdersClient
        """
        return self._raw_client

    async def create(
        self,
        *,
        order: typing.Optional[OrderParams] = OMIT,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateOrderResponse:
        """
        Creates a new [order](entity:Order) that can include information about products for
        purchase and settings to apply to the purchase.

        To pay for a created order, see
        [Pay for Orders](https://developer.squareup.com/docs/orders-api/pay-for-orders).

        You can modify open orders using the [UpdateOrder](api-endpoint:Orders-UpdateOrder) endpoint.

        Parameters
        ----------
        order : typing.Optional[OrderParams]
            The order to create. If this field is set, the only other top-level field that can be
            set is the `idempotency_key`.

        idempotency_key : typing.Optional[str]
            A value you specify that uniquely identifies this
            order among orders you have created.

            If you are unsure whether a particular order was created successfully,
            you can try it again with the same idempotency key without
            worrying about creating duplicate orders.

            For more information, see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateOrderResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.orders.create(
                order={
                    "location_id": "057P5VYJ4A5X1",
                    "reference_id": "my-order-001",
                    "line_items": [
                        {
                            "name": "New York Strip Steak",
                            "quantity": "1",
                            "base_price_money": {"amount": 1599, "currency": "USD"},
                        },
                        {
                            "quantity": "2",
                            "catalog_object_id": "BEMYCSMIJL46OCDV4KYIKXIB",
                            "modifiers": [
                                {"catalog_object_id": "CHQX7Y4KY6N5KINJKZCFURPZ"}
                            ],
                            "applied_discounts": [{"discount_uid": "one-dollar-off"}],
                        },
                    ],
                    "taxes": [
                        {
                            "uid": "state-sales-tax",
                            "name": "State Sales Tax",
                            "percentage": "9",
                            "scope": "ORDER",
                        }
                    ],
                    "discounts": [
                        {
                            "uid": "labor-day-sale",
                            "name": "Labor Day Sale",
                            "percentage": "5",
                            "scope": "ORDER",
                        },
                        {
                            "uid": "membership-discount",
                            "catalog_object_id": "DB7L55ZH2BGWI4H23ULIWOQ7",
                            "scope": "ORDER",
                        },
                        {
                            "uid": "one-dollar-off",
                            "name": "Sale - $1.00 off",
                            "amount_money": {"amount": 100, "currency": "USD"},
                            "scope": "LINE_ITEM",
                        },
                    ],
                },
                idempotency_key="8193148c-9586-11e6-99f9-28cfe92138cf",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.create(
            order=order, idempotency_key=idempotency_key, request_options=request_options
        )
        return response.data

    async def batch_get(
        self,
        *,
        order_ids: typing.Sequence[str],
        location_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BatchGetOrdersResponse:
        """
        Retrieves a set of [orders](entity:Order) by their IDs.

        If a given order ID does not exist, the ID is ignored instead of generating an error.

        Parameters
        ----------
        order_ids : typing.Sequence[str]
            The IDs of the orders to retrieve. A maximum of 100 orders can be retrieved per request.

        location_id : typing.Optional[str]
            The ID of the location for these orders. This field is optional: omit it to retrieve
            orders within the scope of the current authorization's merchant ID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BatchGetOrdersResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.orders.batch_get(
                location_id="057P5VYJ4A5X1",
                order_ids=[
                    "CAISEM82RcpmcFBM0TfOyiHV3es",
                    "CAISENgvlJ6jLWAzERDzjyHVybY",
                ],
            )


        asyncio.run(main())
        """
        response = await self._raw_client.batch_get(
            order_ids=order_ids, location_id=location_id, request_options=request_options
        )
        return response.data

    async def calculate(
        self,
        *,
        order: OrderParams,
        proposed_rewards: typing.Optional[typing.Sequence[OrderRewardParams]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CalculateOrderResponse:
        """
        Enables applications to preview order pricing without creating an order.

        Parameters
        ----------
        order : OrderParams
            The order to be calculated. Expects the entire order, not a sparse update.

        proposed_rewards : typing.Optional[typing.Sequence[OrderRewardParams]]
            Identifies one or more loyalty reward tiers to apply during the order calculation.
            The discounts defined by the reward tiers are added to the order only to preview the
            effect of applying the specified rewards. The rewards do not correspond to actual
            redemptions; that is, no `reward`s are created. Therefore, the reward `id`s are
            random strings used only to reference the reward tier.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CalculateOrderResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.orders.calculate(
                order={
                    "location_id": "D7AVYMEAPJ3A3",
                    "line_items": [
                        {
                            "name": "Item 1",
                            "quantity": "1",
                            "base_price_money": {"amount": 500, "currency": "USD"},
                        },
                        {
                            "name": "Item 2",
                            "quantity": "2",
                            "base_price_money": {"amount": 300, "currency": "USD"},
                        },
                    ],
                    "discounts": [
                        {"name": "50% Off", "percentage": "50", "scope": "ORDER"}
                    ],
                },
            )


        asyncio.run(main())
        """
        response = await self._raw_client.calculate(
            order=order, proposed_rewards=proposed_rewards, request_options=request_options
        )
        return response.data

    async def clone(
        self,
        *,
        order_id: str,
        version: typing.Optional[int] = OMIT,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CloneOrderResponse:
        """
        Creates a new order, in the `DRAFT` state, by duplicating an existing order. The newly created order has
        only the core fields (such as line items, taxes, and discounts) copied from the original order.

        Parameters
        ----------
        order_id : str
            The ID of the order to clone.

        version : typing.Optional[int]
            An optional order version for concurrency protection.

            If a version is provided, it must match the latest stored version of the order to clone.
            If a version is not provided, the API clones the latest version.

        idempotency_key : typing.Optional[str]
            A value you specify that uniquely identifies this clone request.

            If you are unsure whether a particular order was cloned successfully,
            you can reattempt the call with the same idempotency key without
            worrying about creating duplicate cloned orders.
            The originally cloned order is returned.

            For more information, see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CloneOrderResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.orders.clone(
                order_id="ZAISEM52YcpmcWAzERDOyiWS123",
                version=3,
                idempotency_key="UNIQUE_STRING",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.clone(
            order_id=order_id, version=version, idempotency_key=idempotency_key, request_options=request_options
        )
        return response.data

    async def search(
        self,
        *,
        location_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        query: typing.Optional[SearchOrdersQueryParams] = OMIT,
        limit: typing.Optional[int] = OMIT,
        return_entries: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SearchOrdersResponse:
        """
        Search all orders for one or more locations. Orders include all sales,
        returns, and exchanges regardless of how or when they entered the Square
        ecosystem (such as Point of Sale, Invoices, and Connect APIs).

        `SearchOrders` requests need to specify which locations to search and define a
        [SearchOrdersQuery](entity:SearchOrdersQuery) object that controls
        how to sort or filter the results. Your `SearchOrdersQuery` can:

          Set filter criteria.
          Set the sort order.
          Determine whether to return results as complete `Order` objects or as
        [OrderEntry](entity:OrderEntry) objects.

        Note that details for orders processed with Square Point of Sale while in
        offline mode might not be transmitted to Square for up to 72 hours. Offline
        orders have a `created_at` value that reflects the time the order was created,
        not the time it was subsequently transmitted to Square.

        Parameters
        ----------
        location_ids : typing.Optional[typing.Sequence[str]]
            The location IDs for the orders to query. All locations must belong to
            the same merchant.

            Max: 10 location IDs.

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this cursor to retrieve the next set of results for your original query.
            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        query : typing.Optional[SearchOrdersQueryParams]
            Query conditions used to filter or sort the results. Note that when
            retrieving additional pages using a cursor, you must use the original query.

        limit : typing.Optional[int]
            The maximum number of results to be returned in a single page.

            Default: `500`
            Max: `1000`

        return_entries : typing.Optional[bool]
            A Boolean that controls the format of the search results. If `true`,
            `SearchOrders` returns [OrderEntry](entity:OrderEntry) objects. If `false`, `SearchOrders`
            returns complete order objects.

            Default: `false`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SearchOrdersResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.orders.search(
                location_ids=["057P5VYJ4A5X1", "18YC4JDH91E1H"],
                query={
                    "filter": {
                        "state_filter": {"states": ["COMPLETED"]},
                        "date_time_filter": {
                            "closed_at": {
                                "start_at": "2018-03-03T20:00:00+00:00",
                                "end_at": "2019-03-04T21:54:45+00:00",
                            }
                        },
                    },
                    "sort": {"sort_field": "CLOSED_AT", "sort_order": "DESC"},
                },
                limit=3,
                return_entries=True,
            )


        asyncio.run(main())
        """
        response = await self._raw_client.search(
            location_ids=location_ids,
            cursor=cursor,
            query=query,
            limit=limit,
            return_entries=return_entries,
            request_options=request_options,
        )
        return response.data

    async def get(self, order_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetOrderResponse:
        """
        Retrieves an [Order](entity:Order) by ID.

        Parameters
        ----------
        order_id : str
            The ID of the order to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetOrderResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.orders.get(
                order_id="order_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(order_id, request_options=request_options)
        return response.data

    async def update(
        self,
        order_id: str,
        *,
        order: typing.Optional[OrderParams] = OMIT,
        fields_to_clear: typing.Optional[typing.Sequence[str]] = OMIT,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateOrderResponse:
        """
        Updates an open [order](entity:Order) by adding, replacing, or deleting
        fields. Orders with a `COMPLETED` or `CANCELED` state cannot be updated.

        An `UpdateOrder` request requires the following:

        - The `order_id` in the endpoint path, identifying the order to update.
        - The latest `version` of the order to update.
        - The [sparse order](https://developer.squareup.com/docs/orders-api/manage-orders/update-orders#sparse-order-objects)
        containing only the fields to update and the version to which the update is
        being applied.
        - If deleting fields, the [dot notation paths](https://developer.squareup.com/docs/orders-api/manage-orders/update-orders#identifying-fields-to-delete)
        identifying the fields to clear.

        To pay for an order, see
        [Pay for Orders](https://developer.squareup.com/docs/orders-api/pay-for-orders).

        Parameters
        ----------
        order_id : str
            The ID of the order to update.

        order : typing.Optional[OrderParams]
            The [sparse order](https://developer.squareup.com/docs/orders-api/manage-orders/update-orders#sparse-order-objects)
            containing only the fields to update and the version to which the update is
            being applied.

        fields_to_clear : typing.Optional[typing.Sequence[str]]
            The [dot notation paths](https://developer.squareup.com/docs/orders-api/manage-orders/update-orders#identifying-fields-to-delete)
            fields to clear. For example, `line_items[uid].note`.
            For more information, see [Deleting fields](https://developer.squareup.com/docs/orders-api/manage-orders/update-orders#deleting-fields).

        idempotency_key : typing.Optional[str]
            A value you specify that uniquely identifies this update request.

            If you are unsure whether a particular update was applied to an order successfully,
            you can reattempt it with the same idempotency key without
            worrying about creating duplicate updates to the order.
            The latest order version is returned.

            For more information, see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateOrderResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.orders.update(
                order_id="order_id",
                order={
                    "location_id": "location_id",
                    "line_items": [
                        {
                            "uid": "cookie_uid",
                            "name": "COOKIE",
                            "quantity": "2",
                            "base_price_money": {"amount": 200, "currency": "USD"},
                        }
                    ],
                    "version": 1,
                },
                fields_to_clear=["discounts"],
                idempotency_key="UNIQUE_STRING",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.update(
            order_id,
            order=order,
            fields_to_clear=fields_to_clear,
            idempotency_key=idempotency_key,
            request_options=request_options,
        )
        return response.data

    async def pay(
        self,
        order_id: str,
        *,
        idempotency_key: str,
        order_version: typing.Optional[int] = OMIT,
        payment_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayOrderResponse:
        """
        Pay for an [order](entity:Order) using one or more approved [payments](entity:Payment)
        or settle an order with a total of `0`.

        The total of the `payment_ids` listed in the request must be equal to the order
        total. Orders with a total amount of `0` can be marked as paid by specifying an empty
        array of `payment_ids` in the request.

        To be used with `PayOrder`, a payment must:

        - Reference the order by specifying the `order_id` when [creating the payment](api-endpoint:Payments-CreatePayment).
        Any approved payments that reference the same `order_id` not specified in the
        `payment_ids` is canceled.
        - Be approved with [delayed capture](https://developer.squareup.com/docs/payments-api/take-payments/card-payments/delayed-capture).
        Using a delayed capture payment with `PayOrder` completes the approved payment.

        Parameters
        ----------
        order_id : str
            The ID of the order being paid.

        idempotency_key : str
            A value you specify that uniquely identifies this request among requests you have sent. If
            you are unsure whether a particular payment request was completed successfully, you can reattempt
            it with the same idempotency key without worrying about duplicate payments.

            For more information, see [Idempotency](https://developer.squareup.com/docs/working-with-apis/idempotency).

        order_version : typing.Optional[int]
            The version of the order being paid. If not supplied, the latest version will be paid.

        payment_ids : typing.Optional[typing.Sequence[str]]
            The IDs of the [payments](entity:Payment) to collect.
            The payment total must match the order total.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayOrderResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.orders.pay(
                order_id="order_id",
                idempotency_key="c043a359-7ad9-4136-82a9-c3f1d66dcbff",
                payment_ids=[
                    "EnZdNAlWCmfh6Mt5FMNST1o7taB",
                    "0LRiVlbXVwe8ozu4KbZxd12mvaB",
                ],
            )


        asyncio.run(main())
        """
        response = await self._raw_client.pay(
            order_id,
            idempotency_key=idempotency_key,
            order_version=order_version,
            payment_ids=payment_ids,
            request_options=request_options,
        )
        return response.data



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from ..types.payment_options_delay_action import PaymentOptionsDelayAction


class PaymentOptionsParams(typing_extensions.TypedDict):
    autocomplete: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Indicates whether the `Payment` objects created from this `TerminalCheckout` are
    automatically `COMPLETED` or left in an `APPROVED` state for later modification.
    
    Default: true
    """

    delay_duration: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The duration of time after the payment's creation when Square automatically resolves the
    payment. This automatic resolution applies only to payments that do not reach a terminal state
    (`COMPLETED` or `CANCELED`) before the `delay_duration` time period.
    
    This parameter should be specified as a time duration, in RFC 3339 format, with a minimum value
    of 1 minute and a maximum value of 36 hours. This feature is only supported for card payments,
    and all payments will be considered card-present.
    
    This parameter can only be set for a delayed capture payment (`autocomplete=false`). For more
    information, see [Delayed Capture](https://developer.squareup.com/docs/payments-api/take-payments/card-payments/delayed-capture#time-threshold).
    
    Default: "PT36H" (36 hours) from the creation time
    """

    accept_partial_authorization: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    If set to `true` and charging a Square Gift Card, a payment might be returned with
    `amount_money` equal to less than what was requested. For example, a request for $20 when charging
    a Square Gift Card with a balance of $5 results in an APPROVED payment of $5. You might choose
    to prompt the buyer for an additional payment to cover the remainder or cancel the Gift Card
    payment.
    
    This parameter can only be set for a delayed capture payment (`autocomplete=false`).
    
    For more information, see [Take Partial Payments](https://developer.squareup.com/docs/payments-api/take-payments/card-payments/partial-payments-with-gift-cards).
    
    Default: false
    """

    delay_action: typing_extensions.NotRequired[PaymentOptionsDelayAction]
    """
    The action to be applied to the `Payment` when the delay_duration has elapsed.
    The action must be CANCEL or COMPLETE.
    
    The action cannot be set to COMPLETE if an `order_id` is present on the TerminalCheckout.
    
    This parameter can only be set for a delayed capture payment (`autocomplete=false`). For more
    information, see [Delayed Capture](https://developer.squareup.com/docs/payments-api/take-payments/card-payments/delayed-capture#time-threshold).
    
    Default: CANCEL
    See [DelayAction](#type-delayaction) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

import typing

FulfillmentState = typing.Union[
    typing.Literal["PROPOSED", "RESERVED", "PREPARED", "COMPLETED", "CANCELED", "FAILED"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

CatalogItemFoodAndBeverageDetailsDietaryPreferenceStandardDietaryPreference = typing.Union[
    typing.Literal["DAIRY_FREE", "GLUTEN_FREE", "HALAL", "KOSHER", "NUT_FREE", "VEGAN", "VEGETARIAN"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .cash_app_details import CashAppDetailsParams


class DigitalWalletDetailsParams(typing_extensions.TypedDict):
    """
    Additional details about `WALLET` type payments. Contains only non-confidential information.
    """

    status: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The status of the `WALLET` payment. The status can be `AUTHORIZED`, `CAPTURED`, `VOIDED`, or
    `FAILED`.
    """

    brand: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The brand used for the `WALLET` payment. The brand can be `CASH_APP`, `PAYPAY`, `ALIPAY`,
    `RAKUTEN_PAY`, `AU_PAY`, `D_BARAI`, `MERPAY`, `WECHAT_PAY` or `UNKNOWN`.
    """

    cash_app_details: typing_extensions.NotRequired[CashAppDetailsParams]
    """
    Brand-specific details for payments with the `brand` of `CASH_APP`.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .money import MoneyParams


class CheckoutLocationSettingsTippingParams(typing_extensions.TypedDict):
    percentages: typing_extensions.NotRequired[typing.Optional[typing.Sequence[int]]]
    """
    Set three custom percentage amounts that buyers can select at checkout. If Smart Tip is enabled, this only applies to transactions totaling $10 or more.
    """

    smart_tipping_enabled: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Enables Smart Tip Amounts. If Smart Tip Amounts is enabled, tipping works as follows:
    If a transaction is less than $10, the available tipping options include No Tip, $1, $2, or $3.
    If a transaction is $10 or more, the available tipping options include No Tip, 15%, 20%, or 25%. 
    You can set custom percentage amounts with the `percentages` field.
    """

    default_percent: typing_extensions.NotRequired[typing.Optional[int]]
    """
    Set the pre-selected percentage amounts that appear at checkout. If Smart Tip is enabled, this only applies to transactions totaling $10 or more.
    """

    smart_tips: typing_extensions.NotRequired[typing.Optional[typing.Sequence[MoneyParams]]]
    """
    Show the Smart Tip Amounts for this location.
    """

    default_smart_tip: typing_extensions.NotRequired[MoneyParams]
    """
    Set the pre-selected whole amount that appears at checkout when Smart Tip is enabled and the transaction amount is less than $10.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .loyalty_account_mapping import LoyaltyAccountMapping
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class SearchLoyaltyAccountsRequestLoyaltyAccountQuery(UncheckedBaseModel):
    """
    The search criteria for the loyalty accounts.
    """

    mappings: typing.Optional[typing.List[LoyaltyAccountMapping]] = pydantic.Field(default=None)
    """
    The set of mappings to use in the loyalty account search.  
    
    This cannot be combined with `customer_ids`.  
    
    Max: 30 mappings
    """

    customer_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The set of customer IDs to use in the loyalty account search.  
    
    This cannot be combined with `mappings`.  
    
    Max: 30 customer IDs
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class DeviceDetailsParams(typing_extensions.TypedDict):
    """
    Details about the device that took the payment.
    """

    device_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The Square-issued ID of the device.
    """

    device_installation_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The Square-issued installation ID for the device.
    """

    device_name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The name of the device set by the seller.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from ..types.subscription_cadence import SubscriptionCadence
from .money import MoneyParams
from .subscription_pricing import SubscriptionPricingParams


class SubscriptionPhaseParams(typing_extensions.TypedDict):
    """
    Describes a phase in a subscription plan variation. For more information, see [Subscription Plans and Variations](https://developer.squareup.com/docs/subscriptions-api/plans-and-variations).
    """

    uid: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The Square-assigned ID of the subscription phase. This field cannot be changed after a `SubscriptionPhase` is created.
    """

    cadence: SubscriptionCadence
    """
    The billing cadence of the phase. For example, weekly or monthly. This field cannot be changed after a `SubscriptionPhase` is created.
    See [SubscriptionCadence](#type-subscriptioncadence) for possible values
    """

    periods: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The number of `cadence`s the phase lasts. If not set, the phase never ends. Only the last phase can be indefinite. This field cannot be changed after a `SubscriptionPhase` is created.
    """

    recurring_price_money: typing_extensions.NotRequired[MoneyParams]
    """
    The amount to bill for each `cadence`. Failure to specify this field results in a `MISSING_REQUIRED_PARAMETER` error at runtime.
    """

    ordinal: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The position this phase appears in the sequence of phases defined for the plan, indexed from 0. This field cannot be changed after a `SubscriptionPhase` is created.
    """

    pricing: typing_extensions.NotRequired[SubscriptionPricingParams]
    """
    The subscription pricing.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .address import AddressParams


class PrePopulatedDataParams(typing_extensions.TypedDict):
    """
    Describes buyer data to prepopulate in the payment form.
    For more information,
    see [Optional Checkout Configurations](https://developer.squareup.com/docs/checkout-api/optional-checkout-configurations).
    """

    buyer_email: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The buyer email to prepopulate in the payment form.
    """

    buyer_phone_number: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The buyer phone number to prepopulate in the payment form.
    """

    buyer_address: typing_extensions.NotRequired[AddressParams]
    """
    The buyer address to prepopulate in the payment form.
    """



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from ...requests.customer_group import CustomerGroupParams
from ...core.request_options import RequestOptions
from ...core.http_response import HttpResponse
from ...types.create_customer_group_response import CreateCustomerGroupResponse
from ...core.serialization import convert_and_respect_annotation_metadata
from ...core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...types.get_customer_group_response import GetCustomerGroupResponse
from ...core.jsonable_encoder import jsonable_encoder
from ...types.update_customer_group_response import UpdateCustomerGroupResponse
from ...types.delete_customer_group_response import DeleteCustomerGroupResponse
from ...types.add_group_to_customer_response import AddGroupToCustomerResponse
from ...types.remove_group_from_customer_response import RemoveGroupFromCustomerResponse
from ...core.client_wrapper import AsyncClientWrapper
from ...core.http_response import AsyncHttpResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawGroupsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self,
        *,
        group: CustomerGroupParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateCustomerGroupResponse]:
        """
        Creates a new customer group for a business.

        The request must include the `name` value of the group.

        Parameters
        ----------
        group : CustomerGroupParams
            The customer group to create.

        idempotency_key : typing.Optional[str]
            The idempotency key for the request. For more information, see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateCustomerGroupResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/customers/groups",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "group": convert_and_respect_annotation_metadata(
                    object_=group, annotation=CustomerGroupParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateCustomerGroupResponse,
                    construct_type(
                        type_=CreateCustomerGroupResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self, group_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetCustomerGroupResponse]:
        """
        Retrieves a specific customer group as identified by the `group_id` value.

        Parameters
        ----------
        group_id : str
            The ID of the customer group to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetCustomerGroupResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/customers/groups/{jsonable_encoder(group_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetCustomerGroupResponse,
                    construct_type(
                        type_=GetCustomerGroupResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self, group_id: str, *, group: CustomerGroupParams, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[UpdateCustomerGroupResponse]:
        """
        Updates a customer group as identified by the `group_id` value.

        Parameters
        ----------
        group_id : str
            The ID of the customer group to update.

        group : CustomerGroupParams
            The `CustomerGroup` object including all the updates you want to make.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateCustomerGroupResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/customers/groups/{jsonable_encoder(group_id)}",
            method="PUT",
            json={
                "group": convert_and_respect_annotation_metadata(
                    object_=group, annotation=CustomerGroupParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateCustomerGroupResponse,
                    construct_type(
                        type_=UpdateCustomerGroupResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(
        self, group_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[DeleteCustomerGroupResponse]:
        """
        Deletes a customer group as identified by the `group_id` value.

        Parameters
        ----------
        group_id : str
            The ID of the customer group to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DeleteCustomerGroupResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/customers/groups/{jsonable_encoder(group_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteCustomerGroupResponse,
                    construct_type(
                        type_=DeleteCustomerGroupResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def add(
        self, customer_id: str, group_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[AddGroupToCustomerResponse]:
        """
        Adds a group membership to a customer.

        The customer is identified by the `customer_id` value
        and the customer group is identified by the `group_id` value.

        Parameters
        ----------
        customer_id : str
            The ID of the customer to add to a group.

        group_id : str
            The ID of the customer group to add the customer to.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AddGroupToCustomerResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/customers/{jsonable_encoder(customer_id)}/groups/{jsonable_encoder(group_id)}",
            method="PUT",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AddGroupToCustomerResponse,
                    construct_type(
                        type_=AddGroupToCustomerResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def remove(
        self, customer_id: str, group_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[RemoveGroupFromCustomerResponse]:
        """
        Removes a group membership from a customer.

        The customer is identified by the `customer_id` value
        and the customer group is identified by the `group_id` value.

        Parameters
        ----------
        customer_id : str
            The ID of the customer to remove from the group.

        group_id : str
            The ID of the customer group to remove the customer from.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[RemoveGroupFromCustomerResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/customers/{jsonable_encoder(customer_id)}/groups/{jsonable_encoder(group_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RemoveGroupFromCustomerResponse,
                    construct_type(
                        type_=RemoveGroupFromCustomerResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawGroupsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self,
        *,
        group: CustomerGroupParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateCustomerGroupResponse]:
        """
        Creates a new customer group for a business.

        The request must include the `name` value of the group.

        Parameters
        ----------
        group : CustomerGroupParams
            The customer group to create.

        idempotency_key : typing.Optional[str]
            The idempotency key for the request. For more information, see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateCustomerGroupResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/customers/groups",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "group": convert_and_respect_annotation_metadata(
                    object_=group, annotation=CustomerGroupParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateCustomerGroupResponse,
                    construct_type(
                        type_=CreateCustomerGroupResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self, group_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetCustomerGroupResponse]:
        """
        Retrieves a specific customer group as identified by the `group_id` value.

        Parameters
        ----------
        group_id : str
            The ID of the customer group to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetCustomerGroupResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/customers/groups/{jsonable_encoder(group_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetCustomerGroupResponse,
                    construct_type(
                        type_=GetCustomerGroupResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self, group_id: str, *, group: CustomerGroupParams, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[UpdateCustomerGroupResponse]:
        """
        Updates a customer group as identified by the `group_id` value.

        Parameters
        ----------
        group_id : str
            The ID of the customer group to update.

        group : CustomerGroupParams
            The `CustomerGroup` object including all the updates you want to make.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateCustomerGroupResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/customers/groups/{jsonable_encoder(group_id)}",
            method="PUT",
            json={
                "group": convert_and_respect_annotation_metadata(
                    object_=group, annotation=CustomerGroupParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateCustomerGroupResponse,
                    construct_type(
                        type_=UpdateCustomerGroupResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self, group_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[DeleteCustomerGroupResponse]:
        """
        Deletes a customer group as identified by the `group_id` value.

        Parameters
        ----------
        group_id : str
            The ID of the customer group to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DeleteCustomerGroupResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/customers/groups/{jsonable_encoder(group_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteCustomerGroupResponse,
                    construct_type(
                        type_=DeleteCustomerGroupResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def add(
        self, customer_id: str, group_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[AddGroupToCustomerResponse]:
        """
        Adds a group membership to a customer.

        The customer is identified by the `customer_id` value
        and the customer group is identified by the `group_id` value.

        Parameters
        ----------
        customer_id : str
            The ID of the customer to add to a group.

        group_id : str
            The ID of the customer group to add the customer to.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AddGroupToCustomerResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/customers/{jsonable_encoder(customer_id)}/groups/{jsonable_encoder(group_id)}",
            method="PUT",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AddGroupToCustomerResponse,
                    construct_type(
                        type_=AddGroupToCustomerResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def remove(
        self, customer_id: str, group_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[RemoveGroupFromCustomerResponse]:
        """
        Removes a group membership from a customer.

        The customer is identified by the `customer_id` value
        and the customer group is identified by the `group_id` value.

        Parameters
        ----------
        customer_id : str
            The ID of the customer to remove from the group.

        group_id : str
            The ID of the customer group to remove the customer from.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[RemoveGroupFromCustomerResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/customers/{jsonable_encoder(customer_id)}/groups/{jsonable_encoder(group_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RemoveGroupFromCustomerResponse,
                    construct_type(
                        type_=RemoveGroupFromCustomerResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)
